@@ -4,24 +4,14 @@ use warnings;
 use 5.010;
 use Mojo::Base 'Mojo::EventEmitter';
 use WWW::Crawler::Mojo::Job;
+use WWW::Crawler::Mojo::Queue::Memory;
 use WWW::Crawler::Mojo::UserAgent;
+use WWW::Crawler::Mojo::ScraperUtil qw{resolve_href decoded_body};
 use Mojo::Message::Request;
-use Mojo::Util qw{md5_sum xml_escape dumper};
-use List::Util;
-our $VERSION = '0.10';
+use Mojo::Util qw{xml_escape dumper};
+our $VERSION = '0.11';
 
-has active_conn => 0;
-has fix => sub { {} };
-has active_conns_per_host => sub { {} };
-has max_conn => 1;
-has max_conn_per_host => 1;
-has 'peeping_port';
-has peeping_max_length => 30000;
-has queue => sub { [] };
-has 'ua' => sub { WWW::Crawler::Mojo::UserAgent->new };
-has 'ua_name' =>
-    "www-crawler-mojo/$VERSION (+https://github.com/jamadam/www-crawler-mojo)";
-has 'shuffle';
+has clock_speed => 0.25;
 has element_handlers => sub { {
     'script[src]'   => sub { $_[0]->{src} },
     'link[href]'    => sub { $_[0]->{href} },
@@ -63,7 +53,7 @@ has element_handlers => sub { {
             }
         });
         
-        return [$dom->{action},
+        return [$dom->{action} || '',
                     uc ($dom->{method} || 'GET'), Mojo::Parameters->new(%seed)];
     },
     'meta[content]' => sub {
@@ -76,15 +66,25 @@ has element_handlers => sub { {
     },
     '[style]' => sub {
         collect_urls_css(shift->{style});
+    },
+    'urlset[xmlns^=http://www.sitemaps.org/schemas/sitemap/]' => sub {
+        @{$_->find('url loc')->map(sub {$_->content})->to_array};
     }
 } };
+has max_conn => 1;
+has max_conn_per_host => 1;
+has queue => sub { WWW::Crawler::Mojo::Queue::Memory->new };
+has 'shuffle';
+has ua => sub { WWW::Crawler::Mojo::UserAgent->new };
+has ua_name =>
+    "www-crawler-mojo/$VERSION (+https://github.com/jamadam/www-crawler-mojo)";
 
 sub crawl {
     my ($self) = @_;
     
     $self->init;
     
-    die 'No job is given' if (! scalar @{$self->queue});
+    die 'No job is given' if (! $self->queue->length);
     
     $self->emit('start');
     
@@ -100,25 +100,17 @@ sub init {
                                         unless $self->has_subscribers('error');
     $self->on('res', sub { $_[1]->() })
                                         unless $self->has_subscribers('res');
-    $self->on('refer', sub { $_[1]->() })
-                                        unless $self->has_subscribers('refer');
     
     $self->ua->transactor->name($self->ua_name);
     $self->ua->max_redirects(5);
     
-    Mojo::IOLoop->recurring(0.25 => sub {
+    Mojo::IOLoop->recurring($self->clock_speed => sub {
         $self->process_job(@_);
     });
     
-    if ($self->peeping_port) {
-        Mojo::IOLoop->server({port => $self->peeping_port}, sub {
-            $self->peeping_handler(@_);
-        });
-    }
-    
     if ($self->shuffle) {
         Mojo::IOLoop->recurring($self->shuffle => sub {
-            @{$self->{queue}} = List::Util::shuffle @{$self->{queue}};
+            $self->queue->shuffle;
         });
     }
 }
@@ -126,21 +118,21 @@ sub init {
 sub process_job {
     my $self = shift;
     
-    if (!$self->{queue}->[0]) {
-        $self->emit('empty') if (!$self->active_conn);
+    if (!$self->queue->length) {
+        $self->emit('empty') if (!$self->ua->active_conn);
         return;
-    } elsif (!($self->_mod_busyness($self->{queue}->[0]->resolved_uri, 1))) {
+    }
+    if ($self->ua->active_conn >= $self->max_conn ||
+        $self->ua->active_host($self->queue->next->url) >= $self->max_conn_per_host) {
         return;
     }
     
-    my $job = shift @{$self->{queue}};
-    my $uri = $job->resolved_uri;
+    my $job = $self->queue->dequeue;
+    my $url = $job->url;
     my $ua = $self->ua;
-    my $tx = $ua->build_tx($job->method || 'get' => $uri => $job->tx_params);
+    my $tx = $ua->build_tx($job->method || 'get' => $url => $job->tx_params);
     
     $ua->start($tx, sub {
-        $self->_mod_busyness($uri, -1);
-        
         my ($ua, $tx) = @_;
         
         $job->redirect(_urls_redirect($tx));
@@ -153,7 +145,11 @@ sub process_job {
             return;
         }
         
-        $self->emit('res', sub { $self->scrape($res, $job) }, $job, $res);
+        $self->emit('res', sub {
+            $self->scrape($res, $job, $_[0]);
+        }, $job, $res);
+        
+        $job->close;
     });
 }
 
@@ -162,161 +158,83 @@ sub say_start {
     
     print <<"EOF";
 ----------------------------------------
-Crawling is starting with @{[ $self->queue->[0]->resolved_uri ]}
+Crawling is starting with @{[ $self->queue->next->url ]}
 Max Connection  : @{[ $self->max_conn ]}
 User Agent      : @{[ $self->ua_name ]}
-EOF
-
-    print <<"EOF" if ($self->peeping_port);
-Peeping API is available at following URL
-    http://127.0.0.1:@{[ $self->peeping_port ]}/
-EOF
-    
-    print <<"EOF";
 ----------------------------------------
 EOF
 }
 
-sub peeping_handler {
-    my ($self, $loop, $stream) = @_;
-    $stream->on(read => sub {
-        my ($stream, $bytes) = @_;
-        
-        my $path = Mojo::Message::Request->new->parse($bytes)->url->path;
-        
-        if ($path =~ qr{^/queue}) {
-            my $res = sprintf('%s jobs are left.', scalar @{$self->queue});
-            $stream->write("HTTP/1.1 200 OK\n\n");
-            $stream->write($res, sub {shift->close});
-            return;
-        }
-        
-        if ($path =~ qr{^/dumper/(\w+)} && defined $self->{$1}) {
-            my $res = substr(dumper($self->{$1}), 0, $self->peeping_max_length);
-            $stream->write("HTTP/1.1 200 OK\n\n");
-            $stream->write($res, sub {shift->close});
-            return;
-        }
-        
-        $stream->write(
-                    "HTTP/1.1 404 NOT FOUND\n\nNOT FOUND", sub {shift->close});
-    });
-}
-
 sub scrape {
-    my ($self, $res, $job) = @_;
+    my ($self, $res, $job, $cb) = @_;
     
-    return if ($res->code != 200);
+    return unless $res->headers->content_length && $res->body;
     
-    my $base = $job->resolved_uri;
+    my $base = $job->url;
     my $type = $res->headers->content_type;
     
-    if ($type && $type =~ qr{text/(html|xml)}) {
-        if ((my $base_tag = $res->dom->at('base'))) {
+    if ($type && $type =~ qr{^(text|application)/(html|xml|xhtml)}) {
+        if ((my $base_tag = $res->dom->at('base[href]'))) {
             $base = resolve_href($base, $base_tag->attr('href'));
         }
-        my $encode = guess_encoding($res) || 'utf-8';
-        my $dom = Mojo::DOM->new(Encode::decode($encode, $res->body));
+        my $dom = Mojo::DOM->new(decoded_body($res));
         for my $selector (sort keys %{$self->element_handlers}) {
             $dom->find($selector)->each(sub {
                 my $dom = shift;
                 return if ($dom->xml && _wrong_dom_detection($dom));
                 for ($self->element_handlers->{$selector}->($dom)) {
-                    $self->_delegate_enqueue($_, $dom, $job, $base);
+                    $self->_delegate_enqueue($_, $dom, $job, $base, $cb);
                 }
             });
         }
     }
     
     if ($type && $type =~ qr{text/css}) {
-        my $encode  = guess_encoding($res) || 'utf-8';
-        my $body    = Encode::decode($encode, $res->body);
-        for (collect_urls_css($body)) {
-            $self->_delegate_enqueue($_, $job->resolved_uri, $job, $base);
+        for (collect_urls_css(decoded_body($res))) {
+            $self->_delegate_enqueue($_, $job->url, $job, $base, $cb);
         }
     }
 };
 
-sub enqueue {
-    shift->_enqueue([@_]);
-}
-
-sub requeue {
-    shift->_enqueue([@_], 1);
-}
-
-sub _enqueue {
-    my ($self, $jobs, $requeue) = @_;
-    
-    for my $job (@$jobs) {
-        if (! ref $job || ref $job ne 'WWW::Crawler::Mojo::Job') {
-            my $url = !ref $job ? Mojo::URL->new($job) : $job;
-            $job = WWW::Crawler::Mojo::Job->new(resolved_uri => $url);
-        }
-        
-        my $md5_seed = $job->resolved_uri->to_string. ($job->method || '');
-        $md5_seed .= $job->tx_params->to_string if ($job->tx_params);
-        my $md5 = md5_sum($md5_seed);
-        if ($requeue || !exists $self->fix->{$md5}) {
-            $self->fix->{$md5} = undef;
-            push(@{$self->{queue}}, $job);
-        }
-    }
-}
-
-sub _delegate_enqueue {
-    my ($self, $url, $dom, $job, $base) = @_;
-    my $method, my $params;
-    
-    return unless $url;
-    ($url, $method, $params) = @$url if (ref $url);
-    
-    $url = _clean_url_obj($url);
-    my $resolved = resolve_href($base, $url);
-    
-    return unless ($resolved->scheme =~ qr{http|https|ftp|ws|wss});
-    
-    my $child = $job->child(resolved_uri => $resolved, literal_uri => $url);
-    
-    $child->method($method) if $method;
-    
-    if ($params) {
-        if ($method eq 'GET') {
-            $child->resolved_uri->query->append($params);
-        } else {
-            $child->tx_params($params);
-        }
-    }
-    
-    $self->emit('refer', sub { $self->enqueue($_[0] || $child) }, $child, $dom);
-}
-
 sub collect_urls_css {
     map { s/^(['"])// && s/$1$//; $_ } (shift || '') =~ m{url\((.+?)\)}ig;
 }
 
-my $charset_re = qr{\bcharset\s*=\s*['"]?([a-zA-Z0-9_\-]+)['"]?}i;
-
-sub guess_encoding {
-    my $res     = shift;
-    my $type    = $res->headers->content_type;
-    return unless ($type);
-    my $charset = ($type =~ $charset_re)[0];
-    return $charset if ($charset);
-    return _guess_encoding_html($res->body) if ($type =~ qr{text/(html|xml)});
-    return _guess_encoding_css($res->body) if ($type =~ qr{text/css});
+sub _delegate_enqueue {
+    my ($self, $url, $dom, $job, $base, $cb) = @_;
+    my $method, my $params;
+    
+    return unless $url;
+    ($url, $method, $params) = @$url if (ref $url);
+    
+    my $resolved = resolve_href($base, $url);
+    
+    return unless ($resolved->scheme =~ qr{http|https|ftp|ws|wss});
+    
+    my $child = $job->child(url => $resolved, literal_uri => $url);
+    
+    $child->method($method) if $method;
+    
+    if ($params) {
+        if ($method eq 'GET') {
+            $child->url->query->append($params);
+        } else {
+            $child->tx_params($params);
+        }
+    }
+    
+    $cb ||= sub { $_[1]->() };
+    $cb->($self, sub { $self->enqueue($_[0] || $child) }, $child, $dom);
 }
 
-sub resolve_href {
-    my ($base, $href) = @_;
-    $href = ref $href ? $href : Mojo::URL->new($href);
-    $base = ref $base ? $base : Mojo::URL->new($base);
-    my $abs = $href->to_abs($base)->fragment(undef);
-    while ($abs->path->parts->[0] && $abs->path->parts->[0] =~ /^\./) {
-        shift @{$abs->path->parts};
-    }
-    $abs->path->trailing_slash($base->path->trailing_slash) if (!$href->path->to_string);
-    return $abs;
+sub enqueue {
+    my ($self, @jobs) = @_;
+    $self->queue->enqueue(WWW::Crawler::Mojo::Job->upgrade($_)) for @jobs;
+}
+
+sub requeue {
+    my ($self, @jobs) = @_;
+    $self->queue->requeue(WWW::Crawler::Mojo::Job->upgrade($_)) for @jobs;
 }
 
 sub _urls_redirect {
@@ -335,49 +253,6 @@ sub _wrong_dom_detection {
     return;
 }
 
-sub _clean_url_obj {
-    my $url = shift;
-    $url =~ s{^\s*}{}g;
-    $url =~ s{\s*$}{}g;
-    return Mojo::URL->new($url);
-}
-
-sub _guess_encoding_css {
-    return (shift =~ qr{^\s*\@charset ['"](.+?)['"];}is)[0];
-}
-
-sub _guess_encoding_html {
-    my $head = (shift =~ qr{<head>(.+)</head>}is)[0] or return;
-    my $charset;
-    Mojo::DOM->new($head)->find('meta[http\-equiv=Content-Type]')->each(sub{
-        $charset = (shift->{content} =~ $charset_re)[0];
-    });
-    return $charset;
-}
-
-sub _mod_busyness {
-    my ($self, $uri, $inc) = @_;
-    my $key = _host_key($uri);
-    my $hosts = $self->active_conns_per_host;
-    
-    return if ($inc > 0 && ($self->active_conn >= $self->max_conn ||
-                        ($hosts->{$key} || 0) >= $self->max_conn_per_host));
-    
-    $self->{active_conn} += $inc;
-    $hosts->{$key} += $inc;
-    delete($hosts->{$key}) unless ($hosts->{$key});
-    return 1;
-}
-
-sub _host_key {
-    state $well_known_ports = {http => 80, https => 443};
-    my $uri = shift;
-    my $key = $uri->scheme. '://'. $uri->ihost;
-    return $key unless (my $port = $uri->port);
-    $key .= ':'. $port if ($port ne $well_known_ports->{$uri->scheme});
-    return $key;
-}
-
 1;
 
 =head1 NAME
@@ -430,37 +305,12 @@ HTML element handler on scraping.
         return $dom->{src};
     };
 
-=head2 ua
+=head2 clock_speed
 
-A L<Mojo::UserAgent> instance.
+A number of millisecond for main event loop interval. Defaults to 0.25.
 
-    my $ua = $bot->ua;
-    $bot->ua(Mojo::UserAgent->new);
-
-=head2 ua_name
-
-Name of crawler for User-Agent header.
-
-    $bot->ua_name('my-bot/0.01 (+https://example.com/)');
-    say $bot->ua_name; # 'my-bot/0.01 (+https://example.com/)'
-
-=head2 active_conn
-
-A number of current connections.
-
-    $bot->active_conn($bot->active_conn + 1);
-    say $bot->active_conn;
-
-=head2 active_conns_per_host
-
-A number of current connections per host.
-
-    $bot->active_conns_per_host($bot->active_conns_per_host + 1);
-    say $bot->active_conns_per_host;
-
-=head2 fix
-
-A hash whoes keys are md5 hashes of enqueued URLs.
+    $bot->clock_speed(2);
+    my $clock = $bot->clock_speed; # 2
 
 =head2 max_conn
 
@@ -476,27 +326,12 @@ A number of max connections per host.
     $bot->max_conn_per_host(5);
     say $bot->max_conn_per_host; # 5
 
-=head2 peeping_port
-
-An port number for providing peeping monitor. It also evalutated as boolean for
-disabling/enabling the feature. Defaults to undef, meaning disable.
-
-    $bot->peeping_port(3001);
-    say $bot->peeping_port; # 3000
-
-=head2 peeping_max_length
-
-Max length of peeping monitor content.
-
-    $bot->peeping_max_length(100000);
-    say $bot->peeping_max_length; # 100000
-
 =head2 queue
 
-FIFO array contains L<WWW::Crawler::Mojo::Job> objects.
+L<WWW::Crawler::Mojo::Queue::Memory> object for default.
 
-    push(@{$bot->queue}, WWW::Crawler::Mojo::Job->new(...));
-    my $job = shift @{$bot->queue};
+    $bot->queue(WWW::Crawler::Mojo::Queue::Memory->new);
+    $bot->queue->enqueue($job);
 
 =head2 shuffle
 
@@ -506,6 +341,20 @@ for disabling/enabling the feature. Defaults to undef, meaning disable.
     $bot->shuffle(5);
     say $bot->shuffle; # 5
 
+=head2 ua
+
+A L<Mojo::UserAgent> instance.
+
+    my $ua = $bot->ua;
+    $bot->ua(Mojo::UserAgent->new);
+
+=head2 ua_name
+
+Name of crawler for User-Agent header.
+
+    $bot->ua_name('my-bot/0.01 (+https://example.com/)');
+    say $bot->ua_name; # 'my-bot/0.01 (+https://example.com/)'
+
 =head1 EVENTS
 
 L<WWW::Crawler::Mojo> inherits all events from L<Mojo::EventEmitter> and
@@ -518,46 +367,26 @@ Emitted when crawler got response from server. The callback takes 4 arguments.
     $bot->on(res => sub {
         my ($bot, $scrape, $job, $res) = @_;
         if (...) {
-            $scrape->();
+            $scrape->(sub {
+                # called when URL found
+            });
         } else {
             # DO NOTHING
         }
     });
 
-=over
-
-=item $bot
+=head3 $bot
 
 L<WWW::Crawler::Mojo> instance.
 
-=item $scrape
+=head3 $scrape
 
-Scrape URLs out of the document. this is a shorthand of $bot->scrape($job)
+Scraper code reference for current document. The code takes a callback for
+argument in case a URL found.
 
-=item $job
-
-L<WWW::Crawler::Mojo::Job> instance.
-
-=item $res
-
-L<Mojo::Message::Response> instance.
-
-=back
-
-=head2 refer
-
-Emitted when new URI is found. You can enqueue the URI conditionally with
-the callback.
-
-    $bot->on(refer => sub {
+    $scrape(sub {
         my ($bot, $enqueue, $job, $context) = @_;
-        if (...) {
-            $enqueue->();
-        } elseif (...) {
-            $enqueue->(...); # maybe different url
-        } else {
-            # DO NOTHING
-        }
+        ...
     });
 
 =over
@@ -568,7 +397,9 @@ L<WWW::Crawler::Mojo> instance.
 
 =item $enqueue
 
-Scrape URLs out of the document. this is a shorthand of $bot->enqueue($job)
+Enqueue code reference for current URL. This is a shorthand of..
+
+    $bot->enqueue($job)
 
 =item $job
 
@@ -580,6 +411,14 @@ Either L<Mojo::DOM> or L<Mojo::URL> instance.
 
 =back
 
+=head3 $job
+
+L<WWW::Crawler::Mojo::Job> instance.
+
+=head3 $res
+
+L<Mojo::Message::Response> instance.
+
 =head2 empty
 
 Emitted when queue length got zero. The length is checked every 5 seconds.
@@ -643,21 +482,16 @@ Displays starting messages to STDOUT
 
     $bot->say_start;
 
-=head2 peeping_handler
-
-peeping API dispatcher.
-
-    $bot->peeping_handler($loop, $stream);
-
 =head2 scrape
 
 Parses and discovers links in a web page. Each links are appended to FIFO array.
+This performs scraping.
 
-    $bot->scrape($res, $job);
+    $bot->scrape($res, $job, $cb);
 
 =head2 enqueue
 
-Append one or more URIs or L<WWW::Crawler::Mojo::Job> objects.
+Append one or more URLs or L<WWW::Crawler::Mojo::Job> objects.
 
     $bot->enqueue('http://example.com/index1.html');
 
@@ -699,18 +533,6 @@ Collects URLs out of CSS.
 
     @urls = collect_urls_css($dom);
 
-=head2 guess_encoding
-
-Guesses encoding of HTML or CSS with given L<Mojo::Message::Response> instance.
-
-    $encode = WWW::Crawler::Mojo::guess_encoding($res) || 'utf-8'
-
-=head2 resolve_href
-
-Resolves URLs with a base URL.
-
-    WWW::Crawler::Mojo::resolve_href($base, $uri);
-
 =head1 EXAMPLE
 
 L<https://github.com/jamadam/WWW-Flatten>
